from datetime import datetime
from typing import Self

from pydantic import BaseModel, ConfigDict, Field


class MockEmailAddress(BaseModel):
    """Represents a Clerk email address object."""

    model_config = ConfigDict(populate_by_name=True)

    id: str
    email_address: str
    verification: dict | None = None
    linked_to: list[dict] = Field(default_factory=list)

    @classmethod
    def create(cls, email: str, email_id: str) -> Self:
        """Create a verified email address."""

        return cls(
            id=email_id,
            email_address=email,
            verification={"status": "verified", "strategy": "email_code"},
            linked_to=[],
        )


class MockPhoneNumber(BaseModel):
    """Represents a Clerk phone number object."""

    model_config = ConfigDict(populate_by_name=True)

    id: str
    phone_number: str
    verification: dict | None = None
    linked_to: list[dict] = Field(default_factory=list)

    @classmethod
    def create(cls, phone: str, phone_id: str) -> Self:
        """Create a verified phone number."""

        return cls(
            id=phone_id,
            phone_number=phone,
            verification={"status": "verified", "strategy": "phone_code"},
            linked_to=[],
        )


class MockUser(BaseModel):
    """Represents a Clerk User object."""

    model_config = ConfigDict(populate_by_name=True)

    id: str
    external_id: str | None = None
    primary_email_address_id: str | None = None
    primary_phone_number_id: str | None = None
    primary_web3_wallet_id: str | None = None
    username: str | None = None
    first_name: str | None = None
    last_name: str | None = None
    profile_image_url: str = ""
    image_url: str = ""
    has_image: bool = False
    public_metadata: dict = Field(default_factory=dict)
    private_metadata: dict = Field(default_factory=dict)
    unsafe_metadata: dict = Field(default_factory=dict)
    email_addresses: list[MockEmailAddress] = Field(default_factory=list)
    phone_numbers: list[MockPhoneNumber] = Field(default_factory=list)
    web3_wallets: list[dict] = Field(default_factory=list)
    passkeys: list[dict] = Field(default_factory=list)
    password_enabled: bool = False
    two_factor_enabled: bool = False
    totp_enabled: bool = False
    backup_code_enabled: bool = False
    external_accounts: list[dict] = Field(default_factory=list)
    saml_accounts: list[dict] = Field(default_factory=list)
    last_sign_in_at: int | None = None
    banned: bool = False
    locked: bool = False
    lockout_expires_in_seconds: int | None = None
    verification_attempts_remaining: int | None = None
    created_at: int = Field(default_factory=lambda: int(datetime.now().timestamp() * 1000))
    updated_at: int = Field(default_factory=lambda: int(datetime.now().timestamp() * 1000))
    delete_self_enabled: bool = True
    create_organization_enabled: bool = True
    last_active_at: int | None = None
    create_organizations_limit: int | None = None
    legal_accepted_at: int | None = None




from collections.abc import Generator
from contextvars import ContextVar
from contextlib import ExitStack, contextmanager
from typing import Any
from unittest.mock import patch

import pytest

from pytest_clerk_mock.client import MockClerkClient

_current_mock_client: ContextVar[MockClerkClient | None] = ContextVar(
    "_current_mock_client", default=None
)


def _get_current_client() -> MockClerkClient:
    """Get the current MockClerkClient from context."""

    client = _current_mock_client.get()

    if client is None:
        raise RuntimeError("No MockClerkClient is currently active")

    return client


def _mock_authenticate_request(request: Any, options: Any) -> Any:
    """Mock authenticate_request function that delegates to the current mock client."""

    return _get_current_client().authenticate_request(request, options)


class _MockUsersProxy:
    """Proxy that delegates all calls to the current mock client's users."""

    def __getattr__(self, name: str) -> Any:
        return getattr(_get_current_client().users, name)


_users_proxy = _MockUsersProxy()


def _mock_users_class(*args: Any, **kwargs: Any) -> _MockUsersProxy:
    """Mock Users class that returns the proxy."""

    return _users_proxy


def _apply_sdk_patches(stack: ExitStack) -> None:
    """Apply patches to clerk_backend_api SDK internals.

    This patches at the SDK level so it works regardless of when/how
    the Clerk client was instantiated.
    """

    stack.enter_context(
        patch(
            "clerk_backend_api.security.authenticaterequest.authenticate_request",
            _mock_authenticate_request,
        )
    )

    stack.enter_context(
        patch(
            "clerk_backend_api.security.authenticate_request",
            _mock_authenticate_request,
        )
    )

    stack.enter_context(
        patch(
            "clerk_backend_api.sdk.authenticate_request",
            _mock_authenticate_request,
        )
    )

    stack.enter_context(
        patch(
            "clerk_backend_api.users.Users",
            _mock_users_class,
        )
    )


@pytest.fixture
def mock_clerk() -> Generator[MockClerkClient, None, None]:
    """Fixture that provides a mock Clerk client.

    The client is reset after each test to ensure isolation.
    Patches clerk_backend_api SDK internals to intercept all Clerk operations.

    Yields:
        MockClerkClient instance configured with default auth state.

    Example:
        def test_something(mock_clerk):
            # Configure auth state
            mock_clerk.configure_auth("user_123", "org_456")

            # Or use context manager for temporary user switch
            with mock_clerk.as_user("user_456", "org_789"):
                # Test as different user
                pass
    """

    client = MockClerkClient()
    token = _current_mock_client.set(client)

    with ExitStack() as stack:
        _apply_sdk_patches(stack)
        stack.enter_context(patch("clerk_backend_api.Clerk", return_value=client))

        yield client

    _current_mock_client.reset(token)
    client.reset()


@contextmanager
def mock_clerk_backend(
    patch_targets: list[str] | None = None,
    default_user_id: str | None = "user_test_owner",
    default_org_id: str | None = "org_test_123",
    default_org_role: str = "org:admin",
) -> Generator[MockClerkClient, None, None]:
    """Context manager for mocking Clerk backend.

    This provides a moto-like API for mocking Clerk in tests without using fixtures.
    Patches clerk_backend_api SDK internals so it works regardless of when the
    Clerk client was instantiated.

    Args:
        patch_targets: Deprecated. No longer needed - SDK is patched at the internal level.
        default_user_id: Default user ID for authentication (None for unauthenticated)
        default_org_id: Default organization ID
        default_org_role: Default organization role

    Yields:
        MockClerkClient instance

    Example:
        with mock_clerk_backend() as mock:
            mock.configure_auth("user_123", "org_456")
            # Your test code here
    """

    del patch_targets

    client = MockClerkClient(
        default_user_id=default_user_id,
        default_org_id=default_org_id,
        default_org_role=default_org_role,
    )
    token = _current_mock_client.set(client)

    with ExitStack() as stack:
        _apply_sdk_patches(stack)
        stack.enter_context(patch("clerk_backend_api.Clerk", return_value=client))

        yield client

    _current_mock_client.reset(token)
    client.reset()


def create_mock_clerk_fixture(
    patch_targets: list[str] | None = None,
    default_user_id: str | None = "user_test_owner",
    default_org_id: str | None = "org_test_123",
    default_org_role: str = "org:admin",
    autouse: bool = False,
):
    """Factory function to create a mock_clerk fixture with custom configuration.

    Patches clerk_backend_api SDK internals so it works regardless of when the
    Clerk client was instantiated.

    Args:
        patch_targets: Deprecated. No longer needed - SDK is patched at the internal level.
        default_user_id: Default user ID for authentication
        default_org_id: Default organization ID
        default_org_role: Default organization role
        autouse: Whether to automatically use the fixture in all tests

    Returns:
        A pytest fixture function

    Example:
        # In conftest.py
        from pytest_clerk_mock import create_mock_clerk_fixture

        mock_clerk = create_mock_clerk_fixture(autouse=True)
    """

    del patch_targets

    @pytest.fixture(autouse=autouse)
    def custom_mock_clerk() -> Generator[MockClerkClient, None, None]:
        client = MockClerkClient(
            default_user_id=default_user_id,
            default_org_id=default_org_id,
            default_org_role=default_org_role,
        )
        token = _current_mock_client.set(client)

        with ExitStack() as stack:
            _apply_sdk_patches(stack)
            stack.enter_context(patch("clerk_backend_api.Clerk", return_value=client))

            yield client

        _current_mock_client.reset(token)
        client.reset()

    return custom_mock_clerk



from __future__ import annotations

import secrets
from typing import Any
from unittest.mock import MagicMock

import httpx
from clerk_backend_api.models import ClerkErrors
from clerk_backend_api.models.clerkerror import ClerkError
from clerk_backend_api.models.clerkerrors import ClerkErrorsData
from pydantic import BaseModel, Field

from pytest_clerk_mock.models.organization import MockOrganizationMembershipsResponse
from pytest_clerk_mock.models.user import MockEmailAddress, MockPhoneNumber, MockUser

EMAIL_EXISTS_ERROR_CODE = "form_identifier_exists"


class UserNotFoundError(Exception):
    """Raised when a user is not found."""

    def __init__(self, user_id: str) -> None:
        self.user_id = user_id
        super().__init__(f"User not found: {user_id}")


class MockListResponse(BaseModel):
    """Response wrapper for list operations, matching Clerk SDK structure."""

    data: list[MockUser] = Field(default_factory=list)


def _generate_id(prefix: str) -> str:
    """Generate a Clerk-style ID with given prefix."""

    return f"{prefix}_{secrets.token_hex(12)}"


def _create_email_exists_error(email: str) -> ClerkErrors:
    """Create a ClerkErrors exception for duplicate email."""

    mock_response = MagicMock(spec=httpx.Response)
    mock_response.status_code = 422
    mock_response.text = "That email address is taken."
    mock_response.headers = httpx.Headers({})

    return ClerkErrors(
        data=ClerkErrorsData(
            errors=[
                ClerkError(
                    code=EMAIL_EXISTS_ERROR_CODE,
                    message="That email address is taken. Please try another.",
                    long_message="That email address is taken. Please try another.",
                )
            ]
        ),
        raw_response=mock_response,
    )


class MockUsersClient:
    """Mock implementation of Clerk's Users API."""

    def __init__(self) -> None:
        self._users: dict[str, MockUser] = {}
        self._emails: dict[str, str] = {}
        self._memberships: dict[str, MockOrganizationMembershipsResponse] = {}

    def reset(self) -> None:
        """Clear all stored users and email mappings."""

        self._users.clear()
        self._emails.clear()
        self._memberships.clear()

    def create(
        self,
        *,
        email_address: list[str] | None = None,
        phone_number: list[str] | None = None,
        username: str | None = None,
        password: str | None = None,
        first_name: str | None = None,
        last_name: str | None = None,
        external_id: str | None = None,
        public_metadata: dict[str, Any] | None = None,
        private_metadata: dict[str, Any] | None = None,
        unsafe_metadata: dict[str, Any] | None = None,
        skip_password_checks: bool = False,
        skip_password_requirement: bool = False,
        totp_secret: str | None = None,
        backup_codes: list[str] | None = None,
        created_at: str | None = None,
    ) -> MockUser:
        """Create a new user."""

        if email_address:
            for email in email_address:
                if email.lower() in self._emails:
                    raise _create_email_exists_error(email)

        user_id = _generate_id("user")
        email_objects: list[MockEmailAddress] = []
        primary_email_id: str | None = None

        if email_address:
            for i, email in enumerate(email_address):
                email_id = _generate_id("idn")
                email_obj = MockEmailAddress.create(email=email, email_id=email_id)
                email_objects.append(email_obj)
                self._emails[email.lower()] = user_id

                if i == 0:
                    primary_email_id = email_id

        phone_objects: list[MockPhoneNumber] = []
        primary_phone_id: str | None = None

        if phone_number:
            for i, phone in enumerate(phone_number):
                phone_id = _generate_id("idn")
                phone_obj = MockPhoneNumber.create(phone=phone, phone_id=phone_id)
                phone_objects.append(phone_obj)

                if i == 0:
                    primary_phone_id = phone_id

        user = MockUser(
            id=user_id,
            external_id=external_id,
            primary_email_address_id=primary_email_id,
            primary_phone_number_id=primary_phone_id,
            username=username,
            first_name=first_name,
            last_name=last_name,
            email_addresses=email_objects,
            phone_numbers=phone_objects,
            password_enabled=password is not None,
            public_metadata=public_metadata or {},
            private_metadata=private_metadata or {},
            unsafe_metadata=unsafe_metadata or {},
        )

        self._users[user_id] = user

        return user

    def get(self, user_id: str) -> MockUser:
        """Get a user by ID."""

        if user_id not in self._users:
            raise UserNotFoundError(user_id)

        return self._users[user_id]

    def list(
        self,
        *,
        email_address: list[str] | None = None,
        phone_number: list[str] | None = None,
        external_id: list[str] | None = None,
        username: list[str] | None = None,
        user_id: list[str] | None = None,
        query: str | None = None,
        last_active_at_since: int | None = None,
        limit: int = 10,
        offset: int = 0,
        order_by: str = "-created_at",
    ) -> list[MockUser]:
        """List users with optional filters."""

        users = list(self._users.values())

        if email_address:
            email_set = {e.lower() for e in email_address}
            users = [
                u
                for u in users
                if any(e.email_address.lower() in email_set for e in u.email_addresses)
            ]

        if phone_number:
            phone_set = set(phone_number)
            users = [
                u
                for u in users
                if any(p.phone_number in phone_set for p in u.phone_numbers)
            ]

        if external_id:
            ext_id_set = set(external_id)
            users = [u for u in users if u.external_id in ext_id_set]

        if username:
            username_set = set(username)
            users = [u for u in users if u.username in username_set]

        if user_id:
            user_id_set = set(user_id)
            users = [u for u in users if u.id in user_id_set]

        if query:
            query_lower = query.lower()
            users = [
                u
                for u in users
                if (u.first_name and query_lower in u.first_name.lower())
                or (u.last_name and query_lower in u.last_name.lower())
                or (u.username and query_lower in u.username.lower())
                or any(query_lower in e.email_address.lower() for e in u.email_addresses)
            ]

        reverse = order_by.startswith("-")
        sort_key = order_by.lstrip("-+")

        if sort_key == "created_at":
            users.sort(key=lambda u: u.created_at, reverse=reverse)
        elif sort_key == "updated_at":
            users.sort(key=lambda u: u.updated_at, reverse=reverse)

        return users[offset : offset + limit]

    def update(
        self,
        user_id: str,
        *,
        external_id: str | None = None,
        first_name: str | None = None,
        last_name: str | None = None,
        username: str | None = None,
        password: str | None = None,
        primary_email_address_id: str | None = None,
        primary_phone_number_id: str | None = None,
        public_metadata: dict[str, Any] | None = None,
        private_metadata: dict[str, Any] | None = None,
        unsafe_metadata: dict[str, Any] | None = None,
        profile_image_id: str | None = None,
        skip_password_checks: bool = False,
        sign_out_of_other_sessions: bool = False,
        totp_secret: str | None = None,
        backup_codes: list[str] | None = None,
        delete_self_enabled: bool | None = None,
        create_organization_enabled: bool | None = None,
        notify_primary_email_address_changed: bool = False,
    ) -> MockUser:
        """Update a user by ID."""

        if user_id not in self._users:
            raise UserNotFoundError(user_id)

        user = self._users[user_id]
        fields = {
            "external_id": external_id,
            "first_name": first_name,
            "last_name": last_name,
            "username": username,
            "primary_email_address_id": primary_email_address_id,
            "primary_phone_number_id": primary_phone_number_id,
            "public_metadata": public_metadata,
            "private_metadata": private_metadata,
            "unsafe_metadata": unsafe_metadata,
            "delete_self_enabled": delete_self_enabled,
            "create_organization_enabled": create_organization_enabled,
        }
        update_data = {k: v for k, v in fields.items() if v is not None}

        if password is not None:
            update_data["password_enabled"] = True

        updated_user = user.model_copy(update=update_data)
        self._users[user_id] = updated_user

        return updated_user

    def delete(self, user_id: str) -> MockUser:
        """Delete a user by ID."""

        if user_id not in self._users:
            raise UserNotFoundError(user_id)

        user = self._users.pop(user_id)

        for email in user.email_addresses:
            self._emails.pop(email.email_address.lower(), None)

        return user

    def count(
        self,
        *,
        email_address: list[str] | None = None,
        phone_number: list[str] | None = None,
        external_id: list[str] | None = None,
        username: list[str] | None = None,
        user_id: list[str] | None = None,
        query: str | None = None,
    ) -> int:
        """Count users matching the filters."""

        users = self.list(
            email_address=email_address,
            phone_number=phone_number,
            external_id=external_id,
            username=username,
            user_id=user_id,
            query=query,
            limit=999999,
        )

        return len(users)

    def set_organization_memberships(
        self,
        user_id: str,
        memberships: MockOrganizationMembershipsResponse,
    ) -> None:
        """Configure organization memberships for a user."""

        self._memberships[user_id] = memberships

    def get_organization_memberships(
        self,
        user_id: str,
        *,
        limit: int | None = 10,
        offset: int | None = 0,
    ) -> MockOrganizationMembershipsResponse:
        """Get organization memberships for a user (sync version)."""

        return self._memberships.get(
            user_id,
            MockOrganizationMembershipsResponse(data=[], total_count=0),
        )

    async def get_organization_memberships_async(
        self,
        user_id: str,
        *,
        limit: int | None = 10,
        offset: int | None = 0,
    ) -> MockOrganizationMembershipsResponse:
        """Get organization memberships for a user (async version)."""

        return self.get_organization_memberships(user_id, limit=limit, offset=offset)

    async def create_async(
        self,
        *,
        email_address: list[str] | None = None,
        phone_number: list[str] | None = None,
        username: str | None = None,
        password: str | None = None,
        first_name: str | None = None,
        last_name: str | None = None,
        external_id: str | None = None,
        public_metadata: dict[str, Any] | None = None,
        private_metadata: dict[str, Any] | None = None,
        unsafe_metadata: dict[str, Any] | None = None,
        skip_password_checks: bool = False,
        skip_password_requirement: bool = False,
        totp_secret: str | None = None,
        backup_codes: list[str] | None = None,
        created_at: str | None = None,
    ) -> MockUser:
        """Async version of create."""

        return self.create(
            email_address=email_address,
            phone_number=phone_number,
            username=username,
            password=password,
            first_name=first_name,
            last_name=last_name,
            external_id=external_id,
            public_metadata=public_metadata,
            private_metadata=private_metadata,
            unsafe_metadata=unsafe_metadata,
            skip_password_checks=skip_password_checks,
            skip_password_requirement=skip_password_requirement,
            totp_secret=totp_secret,
            backup_codes=backup_codes,
            created_at=created_at,
        )

    async def get_async(self, user_id: str) -> MockUser:
        """Async version of get."""

        return self.get(user_id)

    async def list_async(
        self,
        *,
        email_address: list[str] | None = None,
        phone_number: list[str] | None = None,
        external_id: list[str] | None = None,
        username: list[str] | None = None,
        user_id: list[str] | None = None,
        query: str | None = None,
        last_active_at_since: int | None = None,
        limit: int = 10,
        offset: int = 0,
        order_by: str = "-created_at",
    ) -> MockListResponse:
        """Async version of list.

        Returns a MockListResponse with .data attribute to match Clerk SDK behavior.
        """

        users = self.list(
            email_address=email_address,
            phone_number=phone_number,
            external_id=external_id,
            username=username,
            user_id=user_id,
            query=query,
            last_active_at_since=last_active_at_since,
            limit=limit,
            offset=offset,
            order_by=order_by,
        )

        return MockListResponse(data=users)

    async def update_async(
        self,
        user_id: str,
        *,
        external_id: str | None = None,
        first_name: str | None = None,
        last_name: str | None = None,
        username: str | None = None,
        password: str | None = None,
        primary_email_address_id: str | None = None,
        primary_phone_number_id: str | None = None,
        public_metadata: dict[str, Any] | None = None,
        private_metadata: dict[str, Any] | None = None,
        unsafe_metadata: dict[str, Any] | None = None,
        profile_image_id: str | None = None,
        skip_password_checks: bool = False,
        sign_out_of_other_sessions: bool = False,
        totp_secret: str | None = None,
        backup_codes: list[str] | None = None,
        delete_self_enabled: bool | None = None,
        create_organization_enabled: bool | None = None,
        notify_primary_email_address_changed: bool = False,
    ) -> MockUser:
        """Async version of update."""

        return self.update(
            user_id,
            external_id=external_id,
            first_name=first_name,
            last_name=last_name,
            username=username,
            password=password,
            primary_email_address_id=primary_email_address_id,
            primary_phone_number_id=primary_phone_number_id,
            public_metadata=public_metadata,
            private_metadata=private_metadata,
            unsafe_metadata=unsafe_metadata,
            profile_image_id=profile_image_id,
            skip_password_checks=skip_password_checks,
            sign_out_of_other_sessions=sign_out_of_other_sessions,
            totp_secret=totp_secret,
            backup_codes=backup_codes,
            delete_self_enabled=delete_self_enabled,
            create_organization_enabled=create_organization_enabled,
            notify_primary_email_address_changed=notify_primary_email_address_changed,
        )

    async def delete_async(self, user_id: str) -> MockUser:
        """Async version of delete."""

        return self.delete(user_id)

    async def count_async(
        self,
        *,
        email_address: list[str] | None = None,
        phone_number: list[str] | None = None,
        external_id: list[str] | None = None,
        username: list[str] | None = None,
        user_id: list[str] | None = None,
        query: str | None = None,
    ) -> int:
        """Async version of count."""

        return self.count(
            email_address=email_address,
            phone_number=phone_number,
            external_id=external_id,
            username=username,
            user_id=user_id,
            query=query,
        )



from pytest_clerk_mock.services.auth import AuthSnapshot, MockAuthState
from pytest_clerk_mock.services.users import (
    MockListResponse,
    MockUsersClient,
    UserNotFoundError,
)

__all__ = [
    "AuthSnapshot",
    "MockAuthState",
    "MockListResponse",
    "MockUsersClient",
    "UserNotFoundError",
]



from contextlib import contextmanager
from typing import Generator
from unittest.mock import MagicMock, patch

import httpx
from clerk_backend_api import SDKError
from clerk_backend_api.models import ClerkErrors
from clerk_backend_api.models.clerkerror import ClerkError
from clerk_backend_api.models.clerkerrors import ClerkErrorsData
from pydantic import BaseModel

EMAIL_EXISTS_ERROR_CODE = "form_identifier_exists"


class MockClerkUserResponse(BaseModel):
    """Simple mock Clerk user returned from create_async."""

    id: str


class MockClerkUserListResponse:
    """Mock response from Clerk users.list_async."""

    def __init__(self, data: list[MockClerkUserResponse]):
        self.data = data


@contextmanager
def mock_clerk_user_creation(
    patch_target: str,
    clerk_user_id: str = "user_clerk_mock_123",
) -> Generator[MagicMock, None, None]:
    """Mock Clerk user creation API.

    Args:
        patch_target: The module path to patch.
        clerk_user_id: The ID to return for the created user.

    Yields:
        The mock object for assertions.

    Example:
        with mock_clerk_user_creation("api.core.clerk.clerk.users.create_async", "user_123") as mock:
            # Your test code here
            mock.assert_called_once()
    """

    with patch(patch_target) as mock_create:
        mock_create.return_value = MockClerkUserResponse(id=clerk_user_id)
        yield mock_create


@contextmanager
def mock_clerk_user_creation_failure(
    patch_target: str,
    error_message: str = "Clerk API error",
) -> Generator[MagicMock, None, None]:
    """Mock Clerk user creation API to simulate a failure.

    Args:
        patch_target: The module path to patch.
        error_message: The error message to include.

    Yields:
        The mock object for assertions.

    Example:
        with mock_clerk_user_creation_failure("api.core.clerk.clerk.users.create_async") as mock:
            with pytest.raises(ErrorResponse):
                # Your test code here
    """

    mock_response = MagicMock()
    mock_response.status_code = 500
    mock_response.text = error_message
    mock_response.headers = {}

    with patch(patch_target) as mock_create:
        mock_create.side_effect = SDKError(error_message, mock_response)
        yield mock_create


@contextmanager
def mock_clerk_user_exists(
    create_patch_target: str,
    list_patch_target: str,
    existing_clerk_user_id: str = "user_clerk_existing_123",
) -> Generator[tuple[MagicMock, MagicMock], None, None]:
    """Mock Clerk user creation to simulate an email already exists scenario.

    This simulates the case where create_async fails because the email is taken,
    but list_async returns the existing user so we can link it.

    Args:
        create_patch_target: The module path to patch for create_async.
        list_patch_target: The module path to patch for list_async.
        existing_clerk_user_id: The ID of the existing Clerk user to return.

    Yields:
        A tuple of (mock_create, mock_list) for assertions.

    Example:
        with mock_clerk_user_exists(
            "api.core.clerk.clerk.users.create_async",
            "api.core.clerk.clerk.users.list_async",
            "user_existing_123"
        ) as (mock_create, mock_list):
            # Your test code here
            mock_create.assert_called_once()
            mock_list.assert_called_once()
    """

    mock_response = httpx.Response(
        status_code=422,
        json={
            "errors": [
                {
                    "message": "That email address is taken. Please try another.",
                    "long_message": "That email address is taken. Please try another.",
                    "code": EMAIL_EXISTS_ERROR_CODE,
                }
            ]
        },
    )

    email_exists_error = ClerkErrors(
        data=ClerkErrorsData(
            errors=[
                ClerkError(
                    message="That email address is taken. Please try another.",
                    long_message="That email address is taken. Please try another.",
                    code=EMAIL_EXISTS_ERROR_CODE,
                )
            ]
        ),
        raw_response=mock_response,
    )

    with (
        patch(create_patch_target) as mock_create,
        patch(list_patch_target) as mock_list,
    ):
        mock_create.side_effect = email_exists_error
        mock_list.return_value = MockClerkUserListResponse(
            data=[MockClerkUserResponse(id=existing_clerk_user_id)]
        )
        yield mock_create, mock_list




from pydantic import BaseModel, Field


class MockOrganization(BaseModel):
    """Represents a Clerk Organization."""

    id: str
    name: str = ""
    slug: str = ""
    created_at: int = 0
    updated_at: int = 0


class MockOrganizationMembership(BaseModel):
    """Represents a user's membership in an organization."""

    id: str
    role: str = "org:member"
    organization: MockOrganization | None = None
    public_user_data: dict | None = None
    created_at: int = 0
    updated_at: int = 0


class MockOrganizationMembershipsResponse(BaseModel):
    """Response from get_organization_memberships_async."""

    data: list[MockOrganizationMembership] = Field(default_factory=list)
    total_count: int = 0

